#ifndef PPSSUMM_LOCAL_ALGO
#define PPSSUMM_LOCAL_ALGO
/* 
 * algorithms used by ppssumm which are not specific to ppssumm
 *
 * Tseq(f, n, Os) Append a series of integers (f,f+n] to sequence Os
 * 
 * reorder( I1 , I2 , Or1 , Or2 ) Reorder the elements of sequence (I1,I2]
 * according to the positions read from sequence (Or1,Or2]
 * 
 * sort_order( I1 , I2 , Os ) stable sort of sequence (I1 , I2] also
 * adding a sequence of original positions to Os (ie. *(os+0) = 4 indicates the 
 * element in I1+0 after the sort was originally in position I1+4).
 *
 * find_all ( I1 , I2 , Os , Pred)
 * Find all elements of sequence (I1 , I2] for which pred(*In) returns true.  Add positions (relative to I1) of matching elements to sequence Os.  Matching is by Pred(
 * 
 * find_all ( I1 , I2 , J1 , J2 , Os )
 * Find all elements of sequence (I1 , I2] which are matched by an element in (J1 , J2]. Add positions (relative to I1) of matching elements to sequence Os.
 * 
 * find_all ( I1 , I2 , I1::value_type , Os )
 * Find all elements of sequence (I1 , I2] which are equal to the value provided. Add positions (relative to I1) of matching elements to sequence Os.
 * 
 * equal_ranges( I1 , I2 , Os ) 
 * Add pair<diff , diff> to Os which give the offsets of (I1 , I2] with equal values.
 * (eg. { 0 2 4 6 6 6 7 8 8 } Has equal ranges (3,6] and (7,9] ).
 *
 */
#include <algorithm>
#include <functional>
#include <iterator>
#include <utility>
#include <vector>

/* 
 * Turn a number + count into an iterator sequence
 */
 
template< typename TP , typename Oi_ >
Oi_ Tseq(TP first_ , TP many_ , Oi_ seq_) 
{

	for( ; many_ > 0 ; many_--) 
		*seq_++ = first_++;
	return seq_;
	
}

template< typename Ii_ , typename Io_ >
void reorder( Ii_ first_ , Ii_ last_ , Io_ ofirst_ , Io_ olast_ ) 
{

	/* 
	 * Move sequence element Ii_[n] to position Ii_[*Io_]
	 * for each Ii_ and Io_
	 */
 	if (first_ == last_ || ofirst_ == olast_ ) return;

	typedef typename iterator_traits<Ii_>::difference_type idiff;
	typedef typename iterator_traits<Ii_>::value_type itype;
	typedef typename iterator_traits<Io_>::value_type otype;
	

	idiff diff = last_ - first_;
	//typename iterator_traits<Ii_>::value_type tmp[ static_cast<unsigned int>(diff) ];
	itype tmp[ diff ];

	for(idiff i = 0 
		; i < diff  && ofirst_ != olast_ 
		; i++ , ofirst_++ 
	) {
		*(tmp+i) =   *(first_ + ( static_cast<idiff>(*(ofirst_)) ) );
	}

	for(idiff i = 0 ; i < diff ; i++ ) *(first_+i) = *(tmp+i);
		
}

/*
template< typename Ri_ , typename Oi_ , typename Pred_ >
void find_all( Ri_ first_ , Ri_ last_ , Oi_ order_ , Pred_ pred_ )
{
	// Like sort_order, collect the offsets of all elements in (first_,last_]
	// for which Fn_ comp_ is true
	//
	typename iterator_traits<Ri_>::difference_type i = 0;
	while (first_ != last_) {
		if (pred_(*first_)) order_++ = i;
		i++;
		first_++;
	}
	
}
*/

template< typename Ri_ , typename Oi_ >
void find_all( Ri_ first1_ , Ri_ last1_ , Ri_ first2_ , Ri_ last2_ , Oi_ order_ )
{
	/* 
	 * Like sort_order, collect the offsets of all elements in (first1_,last1_]
	 * which equal an element in (first2_ , last2_)
	 */

	typename std::iterator_traits<Ri_>::difference_type i = 0;
	for ( ; first1_ != last1_ ; first1_++ , i++ ) {

		if ( std::find( first2_ , last2_ , *first1_ ) != last2_ ) {

			order_++ = i;

		}	
	}

	
}

template< typename Ri_ , typename Oi_ >
void find_all( Ri_ first_ , Ri_ last_ , typename iterator_traits<Ri_>::value_type  value_ , Oi_ order_ )
{
	/* 
	 * Like sort_order, collect the offsets of all elements in (first1_,last1_]
	 * which equal an element in (first2_ , last2_)
	 */

	typename std::iterator_traits<Ri_>::difference_type i = 0;
	for ( ; first_ != last_; first_++ , i++ ) {

		if (*first_ == value_) order_++ = i;
	}

	
}


template< typename Ri_ , typename Oi_ > //, typename Fn_ >
void sort_order( Ri_ first_ , Ri_ last_ , Oi_ order_ ) //, Fn_ comp_ )
{
	/* 
	 * Operation: stable_sort of sequence first_ to last_
	 * and build a record of each elements original position relative to first_ (=0)
	 * 
	 * Do this by building a temporary container of pairs and stablesort them on their .first element. 
	 * the cmp_pair comparason below.
	 * 
	 * Assuming position numbers n,m {0 .. N} assign new positions to 
	 * order_[n] = m where m be the original position of the element
	 * such that element[m] should be moved to position n for it
	 * to reach it's final sorted position.
	 *
	 * eg.  Input Sequence:  2 6 1 6 7
	 * 	Becomes       :  1 2 6 6 7
	 * 	giving order_ :  2 0 1 3 4
	 *
	 * The whole point of this is I need the new index ordering to 
	 * re-order other sequences based on the sorting of another
	 *
	 * See the reorder() function below for how to do that.
	 */


	if (first_ == last_) return;

	typedef typename iterator_traits<Ri_>::difference_type idiff;
	typedef typename iterator_traits<Ri_>::value_type itype;
	typedef std::pair< itype , idiff > element;

	typedef std::vector< element > container;


	/* 
	 * Using an array here caused a bus error when optimization
	 * was switched on.  Hence use of container
	 */
	container data;

	idiff pos=0;
	for (Ri_ tmp_ = first_ ; tmp_ != last_ ; tmp_++ , pos++) 
	{
		data.push_back( std::make_pair( *tmp_ , pos ) ) ;
	}

	std::stable_sort( data.begin() , data.end() );

	for ( typename container::const_iterator CI = data.begin()
		; CI != data.end()
		; CI++ , first_++ , order_++
	) {
		*first_ = CI->first;
		*order_ = CI->second;
	}

}


template< typename Ri_ , typename Oi_ > //, typename Fn_ >
void equal_ranges( Ri_ first_ , Ri_ last_  , Oi_ result_)
{

	if (first_ == last_) return;

	typedef std::pair< Ri_ , Ri_ > Eqrange;
	typedef typename std::iterator_traits<Ri_>::difference_type offset;
	typedef std::pair< offset , offset > offset_range;

	Eqrange eqrange = std::make_pair( first_ , last_ );

	while ( eqrange.first != last_ ) {
		
		eqrange = std::equal_range( eqrange.first 
			, last_ 
			, *(eqrange.first) 
		);

		if ( (eqrange.second - eqrange.first) 
		) {
			*result_++ = std::make_pair( eqrange.first - first_
				, eqrange.second - first_
			);
		} 

		eqrange.first = eqrange.second;
	
	}


}

template< typename Ri_ , typename Oi_ > 
void table_sort( Ri_ first_ , Ri_ last_  , Oi_ order_)
{
	
	/* scratch pad
	 *
	 * sort_order first column giving master row_order
	 * make a list of equal ranges.
	 *
	 * for each next column:
	 * 	reorder column based on master row_order
	 * 	stable_sort within equal ranges of previous column
	 * 	reorder range of master row_order defined by equal range using new stable_sort order.
	 * 	find equal ranges from new column within equal ranges of old column.
	 * 	
	 */

	typedef typename std::iterator_traits<Ri_>::value_type ColType;
	typedef typename std::iterator_traits<Ri_>::difference_type offset;
	typedef std::pair< offset , offset > offset_range;


	if (first_ == last_) return;

	ColType col_ = *first_;

	/* sort the first column, re-order others based on this sorting */

	std::vector<offset> row_order;
	sort_order( col_.begin() , col_.end() ,  back_inserter(row_order) );

	/* 
	 * extract equal ranges from first column. These are the ranges which will 
	 * require sorting in subsequent columns
	 */

	vector< offset_range > offsetranges;
	equal_ranges( col_.begin() , col_.end() , back_inserter(offsetranges) );
	
	/* Now the rest of the columns */
	first_++;
	while ( first_ != last_ ) {
		col_ = *first_;

		/* reorder this column in accordance with row_roder */
		reorder( col_.begin() 
			, col_.end() 
			, row_order.begin() 
			, row_order.end() 
		);

		/*
		 * Now sort_order this column over the equal_range's of the previous column.
		 * reorder the master row_order in these equal_range's according to new sub_order.
		 */
		vector< offset_range > newranges;
		for ( typename vector< offset_range >::const_iterator CI = offsetranges.begin()
			; CI != offsetranges.end()
			; CI++
		){
			if ( col_.begin() + CI->first == col_.end()) continue;

			std::vector<unsigned int> sub_order;
			
			sort_order( col_.begin() + CI->first , col_.begin() + CI->second  , std::back_inserter(sub_order) );

			reorder( row_order.begin() + CI->first 
				, row_order.begin() + CI->second 
				, sub_order.begin() 
				, sub_order.end() 
			);

			/* 
			 * Find equal_range's of this column within the equal_ranges defined by previous columns
			 * Remeber, equal ranges in this column outside of these prior equal ranges have been sorted according
			 * to a previous column and so are sorted.
			 */
			
			equal_ranges( col_.begin() + CI->first ,  col_.begin() + CI->second , back_inserter(newranges) );
		}
		offsetranges = newranges;
		first_++;
	}
	
	/* Fill the new column ordering container */
	for (std::vector<unsigned int>::const_iterator CO = row_order.begin() 
		; CO != row_order.end() 
		; CO++ )
		order_++ = *CO;
}


#endif
