#ifndef PPSSUMMSTRING_ALGO
#define PPSSUMMSTRING_ALGO
#include "ppssumm_typedefs.h"
#include <string>
#include <cctype>
#include <sstream>
#include <functional>
#include <iterator>
#include "Dal.h"
#include "errstr.h"


std::string stripdir( std::string );
std::string truncTime( std::string ); 
char toUpper(const char );
char toLower(const char );
std::string asHms (const std::string&  , const int =4 , const double =0.000001 );
std::string asDms (const std::string& , const int =4  , const double =0.000001);

template <typename Ii_>
struct unifylength 
{
	/* 
	 * Originally written to pad all strings in a container to 
	 * be of equal width.  Then converted to apply generally to
	 * a container of containers for which an insert function is
	 * defined.
	 *
	 * There is still a little hardwiring to std::string's because
	 * of asymmetry of stdlib functions.
	 *
	 * Padding is to the size of the largest container or minsize
	 * whicherver is greater.
	 *
	 * frontinsert_ => Add padding to the fronto f the container (rightjutify)
	 * fill 	=> You guessed it.  What to fill with.  Default is container default.
	 */ 

	public:
	typedef typename iterator_traits<Ii_>::value_type ttype;
	typedef typename ttype::size_type tsize;

	tsize minsize;
	bool frontinsert_;
	ttype fill;
	
	unifylength(ttype f = iterator_traits<Ii_>::value_type() 
		, tsize w=0 
		, bool r = false
	) : minsize(w) , frontinsert_(r)  , fill(f)
	{ fill = f; }

	tsize operator()(Ii_ first_ , Ii_ last_)
	{

		/* Length of longest sequence */
		Ii_ tmp_ = first_;
		while (tmp_ != last_) {
			if (minsize < tmp_->size()) minsize = tmp_->size();
			tmp_++;
		}
	
		while ( first_ != last_ ) {

			/* 
			 * We have to insert one at a time via a loop because 
			 * There isn't a std::string::insert( iterator p, size_type n, string s)
			 * to match std::container::insert( iterator p, size_type n, string s)
			 */

			tsize diff = minsize - first_->size();
			//vector<ttype> vfill(diff, fill);

			if ( frontinsert_ ) {

				//first_->insert( first_->begin() , vfill.begin() , vfill.end() );
				for ( ; diff ; diff--) {
					first_->insert( first_->begin() , fill.begin() , fill.end()) ;
				}

			} else {

				//first_->insert( first_->end() , vfill.begin() , vfill.end() );
				for ( ; diff ; diff--) {
					first_->insert( first_->end()  , fill.begin() , fill.end()) ;
				}
			}


			first_++;

		}

		return minsize;
	}
};


/*
 * I know there is a toString in stingutil.h but it doesn't work
 * for bool8 or DString.  It returns ^A (true) and ^@ (false) 
 * and leaves trailing ^@ on the end of strings.
 */


/* 
 * WARNING
 * 
 * <string> with g++ 2.95.3 #define 's CP to be 32
 * Don't use it ! Especially not as a template parameter.
 * Aaaaaaaaaaaaaaaaaaaaargh!
 * 
 */


template< typename TP >
struct toString_ : unary_function< TP , std::string > 
{
	public:
	toString_(int p_=0) : prec_(p_) {}
	std::string operator()( const TP& v_ ) const 
	{
		std::stringstream s;
		if ( prec_ > 0 ) s.precision(prec_);
		s << v_;
		return s.str();
		
	}
	private:
	const int prec_;

};

template<>
struct toString_< std::string > : unary_function< std::string , std::string > 
{
	// Cut a string at first '\0'
	public:
	toString_() : term_('\0') {}
	std::string operator()( const std::string& v_ ) const 
	{
		return v_.substr(0,v_.find_first_of(term_));
	}
	private:
	const char term_;
};

template<>
struct toString_< bool8 > : unary_function< bool8 , std::string > 
{
	// Turn a boolean into text.  Usuallly Y|N, 1|0 , True|False
	public:
	toString_(const char* pT = "Y" , const char* pF ="N" ) : True_(pT) , False_(pF) {}
	std::string operator()( const bool8& v_ ) const 
	{
		return v_ ? True_ : False_ ;
	}
	private:
	const std::string True_;
	const std::string False_;
};


struct notBlank_ : unary_function< std::string  , bool  > 
{
	/* 
	 * Simple function for use with stl algorithms.
	 * Return true if the input string contains elements 
	 * which are not members of ignore_.
	 */
	public:
	notBlank_( const std::string& ign = " \t\n\b\f\r\v" ) : ignore_(ign) {}
	bool operator()( const std::string& s ) const 
	{

		bool rtn;
		if ( s.empty() ) rtn = false;
		else if (ignore_.length()) rtn =  (s.find_first_not_of(ignore_) != std::string::npos) ;
		else rtn = true;

		return rtn;
	}
	private:
	const std::string ignore_;
};

#endif
